@@ -219,6 +219,10 @@ pub struct Action {
     /// production and action.
     pub(crate) prod_num: ProductionIndex,
 
+    /// The relative index of a production within its alternatives.
+    /// Used for auto generation to get a more stable generation experience
+    pub(crate) rel_idx: usize,
+
     /// The function name
     pub(crate) fn_name: String,
 
@@ -246,8 +250,9 @@ impl Display for Action {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::result::Result<(), Error> {
         write!(
             f,
-            "/* {} */ (({}) -> {})  {{ {} }}",
+            "/* {}, {} */ (({}) -> {})  {{ {} }}",
             self.prod_num,
+            self.rel_idx,
             self.args
                 .iter()
                 .map(|a| a.arg_type.type_name())
@@ -357,7 +362,7 @@ impl GrammarTypeInfo {
         )
     }
 
-    fn deduce_type_of_production(&self, prod: &Pr, prod_num: ProductionIndex) -> Result<ASTType> {
+    fn deduce_type_of_production(&self, prod: &Pr, rel_idx: usize) -> Result<ASTType> {
         match prod.effective_len() {
             0 => match prod.2 {
                 ProductionAttribute::None => Ok(ASTType::Unit), // Normal empty production
@@ -368,7 +373,7 @@ impl GrammarTypeInfo {
                     "AddToCollection attribute should not be applied on an empty production"
                 )),
             },
-            _ => Ok(self.struct_data_of_production(prod, prod_num)?),
+            _ => Ok(self.struct_data_of_production(prod, rel_idx)?),
         }
     }
 
@@ -377,18 +382,23 @@ impl GrammarTypeInfo {
         let scanner_state_resolver = grammar_config.get_scanner_state_resolver();
         self.actions = Vec::with_capacity(grammar_config.cfg.pr.len());
         for (i, pr) in grammar_config.cfg.pr.iter().enumerate() {
+            let rel_idx = grammar_config
+                .cfg
+                .get_alternation_index_of_production(i)
+                .unwrap();
             self.actions.push(
                 ActionBuilder::default()
                     .non_terminal(pr.get_n())
                     .prod_num(i)
+                    .rel_idx(rel_idx)
                     .fn_name(NmHlp::to_lower_snake_case(&format!(
                         "{}_{}",
                         pr.get_n_str(),
-                        i
+                        rel_idx
                     )))
                     .prod_string(pr.format(&scanner_state_resolver)?)
                     .args(self.build_argument_list(pr)?)
-                    .out_type(self.deduce_type_of_production(pr, i)?)
+                    .out_type(self.deduce_type_of_production(pr, rel_idx)?)
                     .sem(pr.2.clone())
                     .alts(
                         grammar_config
@@ -416,9 +426,6 @@ impl GrammarTypeInfo {
                     .clone()
                     .with_name(self.actions[actions[0]].non_terminal.clone()),
             ),
-            // Some(ASTType::TypeName(NmHlp::to_upper_camel_case(
-            //     &self.actions[actions[0]].non_terminal,
-            // ))),
             _ => {
                 let actions = actions
                     .iter()
@@ -476,12 +483,12 @@ impl GrammarTypeInfo {
                                     (
                                         NmHlp::to_upper_camel_case(&format!(
                                             "{}_{}",
-                                            a.non_terminal, a.prod_num
+                                            a.non_terminal, a.rel_idx
                                         )),
                                         ASTType::TypeName(format!(
                                             "{}_{}{}",
                                             NmHlp::to_upper_camel_case(nt_ref),
-                                            a.prod_num,
+                                            a.rel_idx,
                                             if cfg[a.prod_num].effective_len() > 0 {
                                                 "<'t>"
                                             } else {
@@ -596,7 +603,7 @@ impl GrammarTypeInfo {
             .for_each(|a| a.adjust_arguments_used(used))
     }
 
-    fn struct_data_of_production(&self, prod: &Pr, prod_num: ProductionIndex) -> Result<ASTType> {
+    fn struct_data_of_production(&self, prod: &Pr, rel_idx: usize) -> Result<ASTType> {
         let mut arguments = self.build_argument_list(prod)?;
         if matches!(prod.2, ProductionAttribute::AddToCollection) {
             Ok(ASTType::Repeat(NmHlp::to_upper_camel_case(
@@ -604,7 +611,7 @@ impl GrammarTypeInfo {
             )))
         } else {
             Ok(ASTType::Struct(
-                NmHlp::to_upper_camel_case(&format!("{}{}", prod.get_n_str(), prod_num)),
+                NmHlp::to_upper_camel_case(&format!("{}{}", prod.get_n_str(), rel_idx)),
                 arguments
                     .drain(..)
                     .map(|arg| (arg.name, arg.arg_type))
@@ -737,11 +744,11 @@ mod tests {
 
     #[test]
     fn test_presentation_of_type_info_1() {
-        let expected = r#"/* 0 */ ((Token<'t> /* a */, Vec<SList<'t>>, Token<'t> /* c */, Vec<SList1<'t>>) -> struct S0 { a_0: Token<'t> /* a */, s_list_1: Vec<SList<'t>>, c_2: Token<'t> /* c */, s_list1_3: Vec<SList1<'t>> })  { - }
-/* 1 */ ((Token<'t> /* d-rpt */, Vec<SList1<'t>>) -> Vec<SList1<'t>>)  { Vec<T>::Push }
-/* 2 */ (() -> Vec<SList1<'t>>)  { Vec<T>::New }
-/* 3 */ ((Token<'t> /* b-rpt */, Vec<SList<'t>>) -> Vec<SList<'t>>)  { Vec<T>::Push }
-/* 4 */ (() -> Vec<SList<'t>>)  { Vec<T>::New }
+        let expected = r#"/* 0, 0 */ ((Token<'t> /* a */, Vec<SList<'t>>, Token<'t> /* c */, Vec<SList1<'t>>) -> struct S0 { a_0: Token<'t> /* a */, s_list_1: Vec<SList<'t>>, c_2: Token<'t> /* c */, s_list1_3: Vec<SList1<'t>> })  { - }
+/* 1, 0 */ ((Token<'t> /* d-rpt */, Vec<SList1<'t>>) -> Vec<SList1<'t>>)  { Vec<T>::Push }
+/* 2, 1 */ (() -> Vec<SList1<'t>>)  { Vec<T>::New }
+/* 3, 0 */ ((Token<'t> /* b-rpt */, Vec<SList<'t>>) -> Vec<SList<'t>>)  { Vec<T>::Push }
+/* 4, 1 */ (() -> Vec<SList<'t>>)  { Vec<T>::New }
 
 S:  struct S { a_0: Token<'t> /* a */, s_list_1: Vec<SList<'t>>, c_2: Token<'t> /* c */, s_list1_3: Vec<SList1<'t>> }
 SList:  struct SList { b_minus_rpt_0: Token<'t> /* b-rpt */ }
@@ -760,18 +767,18 @@ enum ASTType { S(S<'t>), SList(Vec<SList<'t>>), SList1(Vec<SList1<'t>>) }
 
     #[test]
     fn test_presentation_of_type_info_2() {
-        let expected = r#"/* 0 */ ((Token<'t> /* a */, Box<SSuffix2<'t>>) -> struct S0 { a_0: Token<'t> /* a */, s_suffix2_1: Box<SSuffix2<'t>> })  { - }
-/* 1 */ ((Token<'t> /* c */, Box<SSuffix1<'t>>) -> struct SSuffix21 { c_0: Token<'t> /* c */, s_suffix1_1: Box<SSuffix1<'t>> })  { - }
-/* 2 */ ((Token<'t> /* b-opt */, Token<'t> /* c */, Box<SSuffix<'t>>) -> struct SSuffix22 { b_minus_opt_0: Token<'t> /* b-opt */, c_1: Token<'t> /* c */, s_suffix_2: Box<SSuffix<'t>> })  { - }
-/* 3 */ ((Token<'t> /* d-opt */) -> struct SSuffix13 { d_minus_opt_0: Token<'t> /* d-opt */ })  { - }
-/* 4 */ (() -> ())  { - }
-/* 5 */ ((Token<'t> /* d-opt */) -> struct SSuffix5 { d_minus_opt_0: Token<'t> /* d-opt */ })  { - }
-/* 6 */ (() -> ())  { - }
+        let expected = r#"/* 0, 0 */ ((Token<'t> /* a */, Box<SSuffix2<'t>>) -> struct S0 { a_0: Token<'t> /* a */, s_suffix2_1: Box<SSuffix2<'t>> })  { - }
+/* 1, 0 */ ((Token<'t> /* c */, Box<SSuffix1<'t>>) -> struct SSuffix20 { c_0: Token<'t> /* c */, s_suffix1_1: Box<SSuffix1<'t>> })  { - }
+/* 2, 1 */ ((Token<'t> /* b-opt */, Token<'t> /* c */, Box<SSuffix<'t>>) -> struct SSuffix21 { b_minus_opt_0: Token<'t> /* b-opt */, c_1: Token<'t> /* c */, s_suffix_2: Box<SSuffix<'t>> })  { - }
+/* 3, 0 */ ((Token<'t> /* d-opt */) -> struct SSuffix10 { d_minus_opt_0: Token<'t> /* d-opt */ })  { - }
+/* 4, 1 */ (() -> ())  { - }
+/* 5, 0 */ ((Token<'t> /* d-opt */) -> struct SSuffix0 { d_minus_opt_0: Token<'t> /* d-opt */ })  { - }
+/* 6, 1 */ (() -> ())  { - }
 
 S:  struct S { a_0: Token<'t> /* a */, s_suffix2_1: Box<SSuffix2<'t>> }
-SSuffix:  enum SSuffix { SSuffix5(SSuffix_5<'t>), SSuffix6(SSuffix_6) }
-SSuffix1:  enum SSuffix1 { SSuffix1_3(SSuffix1_3<'t>), SSuffix1_4(SSuffix1_4) }
-SSuffix2:  enum SSuffix2 { SSuffix2_1(SSuffix2_1<'t>), SSuffix2_2(SSuffix2_2<'t>) }
+SSuffix:  enum SSuffix { SSuffix0(SSuffix_0<'t>), SSuffix1(SSuffix_1) }
+SSuffix1:  enum SSuffix1 { SSuffix1_0(SSuffix1_0<'t>), SSuffix1_1(SSuffix1_1) }
+SSuffix2:  enum SSuffix2 { SSuffix2_0(SSuffix2_0<'t>), SSuffix2_1(SSuffix2_1<'t>) }
 
 enum ASTType { S(S<'t>), SSuffix(SSuffix<'t>), SSuffix1(SSuffix1<'t>), SSuffix2(SSuffix2<'t>) }
 "#;
