@@ -1,4 +1,6 @@
+use crate::grammar::{Decorate, ProductionAttribute, SymbolAttribute};
 use crate::{Symbol, Terminal};
+use miette::{IntoDiagnostic, Result};
 use std::fmt::{Debug, Display, Error, Formatter};
 use std::hash::Hash;
 use std::ops::Index;
@@ -21,7 +23,7 @@ pub type Rhs = Vec<Symbol>;
 /// Production type
 ///
 #[derive(Debug, Clone, Hash, Eq, PartialEq, Ord, PartialOrd, Serialize, Deserialize)]
-pub struct Pr(pub Symbol, pub Rhs);
+pub struct Pr(pub Symbol, pub Rhs, pub ProductionAttribute);
 
 impl Display for Pr {
     ///
@@ -62,7 +64,11 @@ impl Display for Pr {
 
 impl Default for Pr {
     fn default() -> Self {
-        Self(Symbol::N("".to_owned()), Rhs::default())
+        Self(
+            Symbol::n(""),
+            Rhs::default(),
+            ProductionAttribute::default(),
+        )
     }
 }
 
@@ -72,7 +78,7 @@ impl Pr {
         if !r.iter().all(Self::is_allowed_symbol) {
             panic!("Unexpected symbol kind!");
         }
-        Self(Symbol::N(n.to_owned()), r)
+        Self(Symbol::n(n), r, ProductionAttribute::default())
     }
 
     /// Returns a clone of the non-terminal
@@ -97,7 +103,7 @@ impl Pr {
 
     /// Sets the non-terminal
     pub fn set_n(&mut self, n: String) {
-        self.0 = Symbol::N(n);
+        self.0 = Symbol::N(n, SymbolAttribute::default());
     }
 
     /// Checks if [Rhs] is empty
@@ -114,22 +120,45 @@ impl Pr {
         !(matches!(s, Symbol::T(Terminal::Eps)))
     }
 
+    /// Returns the length of [Rhs] while counting only parser relevant symbols
+    pub fn effective_len(&self) -> usize {
+        self.1.iter().fold(0, |count, s| {
+            if s.is_t() || s.is_n() {
+                count + 1
+            } else {
+                count
+            }
+        })
+    }
+
     /// Formats self with the help of a scanner state resolver
-    pub fn format<R>(&self, scanner_state_resolver: &R) -> String
+    pub fn format<R>(&self, scanner_state_resolver: &R) -> Result<String>
     where
         R: Fn(&[usize]) -> String,
     {
-        format!(
-            "{}: {};",
-            self.0,
-            self.1
-                .iter()
-                .fold(Vec::new(), |mut acc, s| {
-                    acc.push(s.format(scanner_state_resolver));
-                    acc
-                })
-                .join(" ")
-        )
+        let mut s = String::new();
+        self.2
+            .decorate(
+                &mut s,
+                &format!(
+                    "{}: {};",
+                    self.0,
+                    self.1
+                        .iter()
+                        .fold(Ok(Vec::new()), |acc: Result<Vec<String>>, s| {
+                            if let Ok(mut acc) = acc {
+                                acc.push(s.format(scanner_state_resolver)?);
+                                Ok(acc)
+                            } else {
+                                acc
+                            }
+                        })
+                        .map(|v| v.join(" "))?
+                ),
+            )
+            .into_diagnostic()?;
+
+        Ok(s)
     }
 }
 
